
#include <windows.h>
#include "../../../plugin.h"
#include "volume.h"
#include "resource.h"

BOOL Init(void);
void Quit(void);
void Config(PREFS* prefs, CONFIGPARAM* configparam);
void Command(COMMANDPARAM* param);
void Description(COMMANDARGV* argv, LPWSTR lpszText, int nLength);

PLUGIN plugin = {
	PLUGIN_VERSION,
	0,
	L"volume",
	NULL,
	NULL,
	Init,
	Quit,
	Config,
	Command,
	Description
};

EXTERN_C PLUGIN * WINAPI GetPlugin(void)
{
	return &plugin;
}

INT_PTR CALLBACK DlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static CONFIGPARAM* configparam;

	switch(msg)
	{
	case WM_INITDIALOG:
		configparam = (CONFIGPARAM*)lParam;
		if(!configparam->bInitialized)
		{
			configparam->argv.nValue[0] = 0;
			configparam->argv.nValue[1] = 5;
		}
		if(configparam->argv.nValue[0]==0)
			SendDlgItemMessage(hwnd, IDC_RADIO_UP, BM_SETCHECK, 1, 0);
		if(configparam->argv.nValue[0]==1)
			SendDlgItemMessage(hwnd, IDC_RADIO_DOWN, BM_SETCHECK, 1, 0);
		if(configparam->argv.nValue[0]==2)
			SendDlgItemMessage(hwnd, IDC_RADIO_MUTE, BM_SETCHECK, 1, 0);
		SetDlgItemInt(hwnd, IDC_EDIT1, configparam->argv.nValue[1], TRUE);
		EnableWindow(GetDlgItem(hwnd, IDC_EDIT1), SendDlgItemMessage(hwnd, IDC_RADIO_MUTE, BM_GETCHECK, 0, 0)?FALSE:TRUE);
		break;
	case WM_NOTIFY:
		switch(((LPNMHDR)lParam)->code)
		{
		case PSN_APPLY:
			if(SendDlgItemMessage(hwnd, IDC_RADIO_UP, BM_GETCHECK, 0, 0))
				configparam->argv.nValue[0] = 0;
			if(SendDlgItemMessage(hwnd, IDC_RADIO_DOWN, BM_GETCHECK, 0, 0))
				configparam->argv.nValue[0] = 1;
			if(SendDlgItemMessage(hwnd, IDC_RADIO_MUTE, BM_GETCHECK, 0, 0))
				configparam->argv.nValue[0] = 2;
			configparam->argv.nValue[1] = GetDlgItemInt(hwnd, IDC_EDIT1, NULL, TRUE);
			break;
		}
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_RADIO_UP:
		case IDC_RADIO_DOWN:
		case IDC_RADIO_MUTE:
			if(HIWORD(wParam)==BN_CLICKED)
			{
				EnableWindow(GetDlgItem(hwnd, IDC_EDIT1), SendDlgItemMessage(hwnd, IDC_RADIO_MUTE, BM_GETCHECK, 0, 0)?FALSE:TRUE);
				PropSheet_Changed(GetParent(hwnd), hwnd);
			}
			break;
		case IDC_EDIT1:
			if(HIWORD(wParam)==EN_CHANGE)
				PropSheet_Changed(GetParent(hwnd), hwnd);
			break;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

BOOL Init(void)
{
	return TRUE;
}

void Quit(void)
{
}

void Config(PREFS* prefs, CONFIGPARAM* configparam)
{
	prefs->hInstance = plugin.hDllInstance;
	prefs->id = IDD_DIALOG;
	prefs->DlgProc = DlgProc;
	prefs->lParam = (LPARAM)configparam;
}

void Command(COMMANDPARAM* param)
{
	float fLevel;
	float fStep;
	BOOL bMute;

	switch(param->argv.nValue[0])
	{
	case 0:
	case 1:
		fStep = (float)param->argv.nValue[1]/100;
		if(fStep<0) fStep = 0;
		if(fStep>1) fStep = 1;
		GetVolume(&fLevel);
		if(param->argv.nValue[0]==0)
			fLevel += fStep;
		if(param->argv.nValue[0]==1)
			fLevel -= fStep;
		if(fLevel<0) fLevel = 0;
		if(fLevel>1) fLevel = 1;
		SetVolume(fLevel);
		break;
	case 2:
		if(GetMute(&bMute))
			SetMute(bMute?FALSE:TRUE);
		break;
	}
}

void Description(COMMANDARGV* argv, LPWSTR lpszText, int nLength)
{
	WCHAR szText[1024];

	if(argv->nValue[0]==0)
		wsprintfW(szText, L"ʂグ : %d%%", argv->nValue[1]);
	if(argv->nValue[0]==1)
		wsprintfW(szText, L"ʂ : %d%%", argv->nValue[1]);
	if(argv->nValue[0]==2)
		wsprintfW(szText, L"~[gON/OFF");
	lstrcpynW(lpszText, szText, nLength);
}
