
#pragma comment(lib, "winmm.lib")

#include <windows.h>
#include <objbase.h>
#include <mmdeviceapi.h>
#include <EndpointVolume.h>

#include "volume.h"

template <class T> void SafeRelease(T **ppT)
{
    if (*ppT)
    {
        (*ppT)->Release();
        *ppT = NULL;
    }
}

BOOL audio_Open(IAudioEndpointVolume **pEndpointVolume)
{
	HRESULT hr;
	IMMDeviceEnumerator *deviceEnumerator = NULL;
	IMMDevice *device = NULL;
	IAudioEndpointVolume *endpointVolume = NULL;

	hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&deviceEnumerator));
	if (FAILED(hr))
	{
		return FALSE;
	}
	hr = deviceEnumerator->GetDefaultAudioEndpoint(eRender, eConsole, &device);
	if (FAILED(hr))
	{
		SafeRelease(&deviceEnumerator);
		return FALSE;
	}
	hr = device->Activate(__uuidof(IAudioEndpointVolume), CLSCTX_INPROC_SERVER, NULL, reinterpret_cast<void **>(&endpointVolume));
	if (FAILED(hr))
	{
		SafeRelease(&deviceEnumerator);
		SafeRelease(&device);
		return FALSE;
	}
	SafeRelease(&deviceEnumerator);
	SafeRelease(&device);
	*pEndpointVolume = endpointVolume;
	return TRUE;
}

void audio_Close(IAudioEndpointVolume *pEndpointVolume)
{
	SafeRelease(&pEndpointVolume);
}

BOOL audio_GetVolume(float *fLevel)
{
	IAudioEndpointVolume *endpointVolume;
	HRESULT hr;

	if(audio_Open(&endpointVolume))
	{
		hr = endpointVolume->GetMasterVolumeLevelScalar(fLevel);
		if (FAILED(hr))
		{
			audio_Close(endpointVolume);
			return FALSE;
		}
		audio_Close(endpointVolume);
		return TRUE;
	}
	return FALSE;
}

BOOL audio_SetVolume(float fLevel)
{
	IAudioEndpointVolume *endpointVolume;
	HRESULT hr;

	if(audio_Open(&endpointVolume))
	{
		hr = endpointVolume->SetMasterVolumeLevelScalar(fLevel, NULL);
		if (FAILED(hr))
		{
			audio_Close(endpointVolume);
			return FALSE;
		}
		audio_Close(endpointVolume);
		return TRUE;
	}
	return FALSE;
}

BOOL audio_GetMute(BOOL *bMute)
{
	IAudioEndpointVolume *endpointVolume;

	if(audio_Open(&endpointVolume))
	{
		endpointVolume->GetMute(bMute);
		audio_Close(endpointVolume);
		return TRUE;
	}
	return FALSE;
}

BOOL audio_SetMute(BOOL bMute)
{
	IAudioEndpointVolume *endpointVolume;

	if(audio_Open(&endpointVolume))
	{
		endpointVolume->SetMute(bMute, NULL);
		audio_Close(endpointVolume);
		return TRUE;
	}
	return FALSE;
}
//----
BOOL GetVolumeMixerControl(HMIXER hMixer, MIXERCONTROL *mxcVolume)
{
	MIXERLINE mxl;
	MIXERLINECONTROLS mxlc;

	mxl.cbStruct        = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if(mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl,
		MIXER_OBJECTF_HMIXER|MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR)
		return FALSE;

	mxlc.cbStruct      = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID      = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
	mxlc.cControls     = 1;
	mxlc.cbmxctrl      = sizeof(MIXERCONTROL);
	mxlc.pamxctrl      = mxcVolume;
	if(mixerGetLineControls((HMIXEROBJ)hMixer, &mxlc,
		MIXER_OBJECTF_HMIXER|MIXER_GETLINECONTROLSF_ONEBYTYPE)!=MMSYSERR_NOERROR)
		return FALSE;

	return TRUE;
}

BOOL GetMuteMixerControl(HMIXER hMixer, MIXERCONTROL *mxcMute)
{
	MIXERLINE mxl;
	MIXERLINECONTROLS mxlc;

	mxl.cbStruct        = sizeof(MIXERLINE);
	mxl.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_SPEAKERS;
	if(mixerGetLineInfo((HMIXEROBJ)hMixer, &mxl,
		MIXER_OBJECTF_HMIXER|MIXER_GETLINEINFOF_COMPONENTTYPE)!=MMSYSERR_NOERROR)
		return FALSE;

	mxlc.cbStruct      = sizeof(MIXERLINECONTROLS);
	mxlc.dwLineID      = mxl.dwLineID;
	mxlc.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
	mxlc.cControls     = 1;
	mxlc.cbmxctrl      = sizeof(MIXERCONTROL);
	mxlc.pamxctrl      = mxcMute;
	if(mixerGetLineControls((HMIXEROBJ)hMixer, &mxlc,
		MIXER_OBJECTF_HMIXER|MIXER_GETLINECONTROLSF_ONEBYTYPE)!=MMSYSERR_NOERROR)
		return FALSE;

	return TRUE;
}

BOOL mixer_Open(HMIXER *hMixer)
{
	if(mixerOpen(hMixer, 0, 0, 0, MIXER_OBJECTF_MIXER)!=MMSYSERR_NOERROR)
		return FALSE;
	return TRUE;
}

BOOL mixer_Close(HMIXER hMixer)
{
	mixerClose(hMixer);
	return TRUE;
}

BOOL mixer_GetVolume(float *fLevel)
{
	HMIXER hMixer;

	if(mixer_Open(&hMixer))
	{
		MIXERCONTROL mxcVolume;
		MIXERCONTROLDETAILS mxcd;
		MIXERCONTROLDETAILS_UNSIGNED mxcdVolume;

		GetVolumeMixerControl(hMixer, &mxcVolume);

		mxcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);
		mxcd.dwControlID    = mxcVolume.dwControlID;
		mxcd.cChannels      = 1;
		mxcd.cMultipleItems = 0;
		mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
		mxcd.paDetails      = &mxcdVolume;

		if(mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
			MIXER_OBJECTF_HMIXER|MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
		{
			mixer_Close(hMixer);
			return FALSE;
		}

		int nLevel = ((mxcdVolume.dwValue - mxcVolume.Bounds.dwMinimum) * 100 +
			(mxcVolume.Bounds.dwMaximum - mxcVolume.Bounds.dwMinimum) / 2) /
				(mxcVolume.Bounds.dwMaximum - mxcVolume.Bounds.dwMinimum);
		*fLevel = (float)nLevel/100;

		mixer_Close(hMixer);
		return TRUE;
	}
	return FALSE;
}

BOOL mixer_SetVolume(float fLevel)
{
	HMIXER hMixer;

	if(mixer_Open(&hMixer))
	{
		MIXERCONTROL mxcVolume;
		MIXERCONTROLDETAILS mxcd;
		MIXERCONTROLDETAILS_UNSIGNED mxcdVolume;

		GetVolumeMixerControl(hMixer, &mxcVolume);

		mxcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);
		mxcd.dwControlID    = mxcVolume.dwControlID;
		mxcd.cChannels      = 1;
		mxcd.cMultipleItems = 0;
		mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
		mxcd.paDetails      = &mxcdVolume;

		mxcdVolume.dwValue = (DWORD)(fLevel*(mxcVolume.Bounds.dwMaximum-mxcVolume.Bounds.dwMinimum)+mxcVolume.Bounds.dwMinimum);

		if(mixerSetControlDetails((HMIXEROBJ)hMixer, &mxcd,
			MIXER_OBJECTF_HMIXER|MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
		{
			mixer_Close(hMixer);
			return FALSE;
		}

		mixer_Close(hMixer);
		return TRUE;
	}
	return FALSE;
}

BOOL mixer_GetMute(BOOL *bMute)
{
	HMIXER hMixer;

	if(mixer_Open(&hMixer))
	{
		MIXERCONTROL mxcMute;
		MIXERCONTROLDETAILS mxcd;
		MIXERCONTROLDETAILS_BOOLEAN mxcdMute;

		GetMuteMixerControl(hMixer, &mxcMute);

		mxcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);
		mxcd.dwControlID    = mxcMute.dwControlID;
		mxcd.cChannels      = 1;
		mxcd.cMultipleItems = 0;
		mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
		mxcd.paDetails      = &mxcdMute;

		if(mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
			MIXER_OBJECTF_HMIXER|MIXER_GETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
		{
			mixer_Close(hMixer);
			return FALSE;
		}

		*bMute = mxcdMute.fValue;
		mixer_Close(hMixer);
		return TRUE;
	}
	return FALSE;
}

BOOL mixer_SetMute(BOOL bMute)
{
	HMIXER hMixer;

	if(mixer_Open(&hMixer))
	{
		MIXERCONTROL mxcMute;
		MIXERCONTROLDETAILS mxcd;
		MIXERCONTROLDETAILS_BOOLEAN mxcdMute;

		GetMuteMixerControl(hMixer, &mxcMute);

		mxcd.cbStruct       = sizeof(MIXERCONTROLDETAILS);
		mxcd.dwControlID    = mxcMute.dwControlID;
		mxcd.cChannels      = 1;
		mxcd.cMultipleItems = 0;
		mxcd.cbDetails      = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
		mxcd.paDetails      = &mxcdMute;

		mxcdMute.fValue = bMute;

		if(mixerSetControlDetails((HMIXEROBJ)hMixer, &mxcd,
			MIXER_OBJECTF_HMIXER|MIXER_SETCONTROLDETAILSF_VALUE)!=MMSYSERR_NOERROR)
		{
			mixer_Close(hMixer);
			return FALSE;
		}

		mixer_Close(hMixer);
		return TRUE;
	}
	return FALSE;
}
//----
BOOL IsVistaOrLater(void)
{
	OSVERSIONINFO ovi = {0};
	ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&ovi);
	return (ovi.dwMajorVersion>=6);
}
//----
BOOL WINAPI GetVolume(float *fLevel)
{
	if(IsVistaOrLater())
		return audio_GetVolume(fLevel);
	return mixer_GetVolume(fLevel);
}

BOOL WINAPI SetVolume(float fLevel)
{
	if(IsVistaOrLater())
		return audio_SetVolume(fLevel);
	return mixer_SetVolume(fLevel);
}

BOOL WINAPI GetMute(BOOL *bMute)
{
	if(IsVistaOrLater())
		return audio_GetMute(bMute);
	return mixer_GetMute(bMute);
}

BOOL WINAPI SetMute(BOOL bMute)
{
	if(IsVistaOrLater())
		return audio_SetMute(bMute);
	return mixer_SetMute(bMute);
}
